<?php
/*
Plugin Name: TNT Custom Header/Footer Updater
Description: Updates head, header, footer, and styles content from a specified static site. Automatically removes static meta tags and title, and replaces local paths with absolute URLs in the header and footer sections. Provides an admin interface for manual updates and content management, including easy copy functionality for header and footer content.
Version: 2.62
Author: tdhelixx
*/

/*
Version 2.62:
- Fixed issue with HTML comment handling to prevent modifying correctly formatted comments
- Improved regex pattern to only fix malformed comments ending with ->
- Removed unnecessary testing components for streamlined operation
- Enhanced comment fixing function to preserve existing properly formatted comments
- Preserved mailto: links by preventing URL modification for special protocols (mailto:, tel:, sms:)

Version 2.60:
- Fixed handling of meta description tags, ensuring case variations are recognized.
- Preserved 3rd-party tracking scripts ending with cctm.xyz/t.js, preventing alteration.
- Retained URLs beginning with // to maintain intended structure.
- Resolved issue with blog links opening in a new tab when returning to the main site.
- Updated footer form to include a full URL during client theme creation and updates.
*/

// Function to inject content into header.php and footer.php
function inject_content_into_files($head_content, $header_content, $footer_content) {
    $header_template_path = get_template_directory() . '/header.php';
    $footer_template_path = get_template_directory() . '/footer.php';

    // Check file permissions
    if (!is_writable($header_template_path) || !is_writable($footer_template_path)) {
        log_tnt_message("Error: Template files are not writable");
        return false;
    }

    // Backup files before modification
    $backup_time = current_time('timestamp');
    $header_backup = $header_template_path . '.backup.' . $backup_time;
    $footer_backup = $footer_template_path . '.backup.' . $backup_time;
    
    if (!copy($header_template_path, $header_backup) || !copy($footer_template_path, $footer_backup)) {
        log_tnt_message("Failed to create backup files");
        return false;
    }

    try {
        $header_template_content = file_get_contents($header_template_path);
        $footer_template_content = file_get_contents($footer_template_path);

        if ($header_template_content === false || $footer_template_content === false) {
            throw new Exception("Failed to read template files");
        }

        $head_injection_start = '<!-- Inject Head Content Start -->';
        $head_injection_end = '<!-- Inject Head Content End -->';
        $header_injection_start = '<!-- Inject Header Content Start -->';
        $header_injection_end = '<!-- Inject Header Content End -->';
        $footer_injection_start = '<!-- Inject Footer Content Start -->';
        $footer_injection_end = '<!-- Inject Footer Content End -->';

        // Remove old injected content
        $new_header_content = preg_replace('/' . preg_quote($head_injection_start, '/') . '.*?' . preg_quote($head_injection_end, '/') . '/s', $head_injection_start . PHP_EOL . $head_injection_end, $header_template_content);
        $new_header_content = preg_replace('/' . preg_quote($header_injection_start, '/') . '.*?' . preg_quote($header_injection_end, '/') . '/s', $header_injection_start . PHP_EOL . $header_injection_end, $new_header_content);
        $new_footer_content = preg_replace('/' . preg_quote($footer_injection_start, '/') . '.*?' . preg_quote($footer_injection_end, '/') . '/s', $footer_injection_start . PHP_EOL . $footer_injection_end, $footer_template_content);

        // Add new injected content
        $new_header_content = str_replace(
            $head_injection_start . PHP_EOL . $head_injection_end,
            $head_injection_start . PHP_EOL . $head_content . PHP_EOL . $head_injection_end,
            $new_header_content
        );

        $new_header_content = str_replace(
            $header_injection_start . PHP_EOL . $header_injection_end,
            $header_injection_start . PHP_EOL . $header_content . PHP_EOL . $header_injection_end,
            $new_header_content
        );

        $new_footer_content = str_replace(
            $footer_injection_start . PHP_EOL . $footer_injection_end,
            $footer_injection_start . PHP_EOL . $footer_content . PHP_EOL . $footer_injection_end,
            $new_footer_content
        );

        if (file_put_contents($header_template_path, $new_header_content) === false ||
            file_put_contents($footer_template_path, $new_footer_content) === false) {
            throw new Exception("Failed to write to template files");
        }

        log_tnt_message("Successfully injected content into header.php and footer.php");
        return true;
    } catch (Exception $e) {
        // Restore from backup on failure
        copy($header_backup, $header_template_path);
        copy($footer_backup, $footer_template_path);
        log_tnt_message("Error during injection, restored from backup: " . $e->getMessage());
        return false;
    } finally {
        // Cleanup old backups (keep last 5)
        $backup_files = glob(get_template_directory() . '/*.backup.*');
        if (count($backup_files) > 5) {
            rsort($backup_files);
            $old_backups = array_slice($backup_files, 5);
            foreach ($old_backups as $file) {
                @unlink($file);
            }
        }
    }
}

// Function to fetch and parse the static site's index page
function fetch_static_site_content($url) {
    // Validate URL
    if (!filter_var($url, FILTER_VALIDATE_URL)) {
        log_tnt_message("Invalid URL format: $url");
        return null;
    }

    // Setup context with timeout and user agent
    $context = stream_context_create([
        'http' => [
            'timeout' => 30,
            'user_agent' => 'WordPress/TNT-Header-Footer-Plugin',
            'follow_location' => true,
            'verify_peer' => true,
        ],
        'ssl' => [
            'verify_peer' => true,
            'verify_peer_name' => true,
        ]
    ]);

    try {
        $content = @file_get_contents($url, false, $context);
        if ($content === false) {
            $error = error_get_last();
            log_tnt_message("Failed to fetch content from $url: " . ($error['message'] ?? 'Unknown error'));
            return null;
        }

        // Check if content is empty
        if (empty(trim($content))) {
            log_tnt_message("Empty content received from $url");
            return null;
        }

        $dom = new DOMDocument();
        libxml_use_internal_errors(true); // Suppress DOMDocument warnings
        if (!$dom->loadHTML($content, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD)) {
            $errors = libxml_get_errors();
            libxml_clear_errors();
            log_tnt_message("Failed to parse HTML content from $url: " . print_r($errors, true));
            return null;
        }
        libxml_clear_errors();

        // Log success but limit content preview
        log_tnt_message("Successfully fetched and parsed content from $url");
        
        return $dom;
    } catch (Exception $e) {
        log_tnt_message("Exception while fetching content from $url: " . $e->getMessage());
        return null;
    }
}

// Function to adjust paths in the HTML content
function adjust_paths($content, $base_url) {
    // Ensure base URL ends with a single slash
    $base_url = rtrim($base_url, '/') . '/';

    // Adjust paths for src and href attributes
    $content = preg_replace_callback('/(src|href|action)=["\']([^"\']+)["\']/', function($matches) use ($base_url) {
        $attr = $matches[1];
        $url = $matches[2];
        
        // Trim the URL
        $url = trim($url);

        // 1. Skip empty URLs
        if (empty($url)) {
            return $matches[0];
        }

        // 2. Skip special protocols and patterns
        $skip_patterns = array(
            // Common protocols
            '^mailto:',
            '^tel:',
            '^sms:',
            '^https?:',
            '^ftp:',
            '^data:',
            '^javascript:',
            // Special URLs
            '^#',
            '^\/\/',
            // Third-party scripts
            '\/\d+\.cctm\.xyz\/t\.js$',
            // WordPress specific
            '^wp-',
            '^\/wp-'
        );

        foreach ($skip_patterns as $pattern) {
            if (preg_match('/' . $pattern . '/', $url)) {
                return $matches[0];
            }
        }

        // 3. Handle relative paths
        if (strpos($url, '/') === 0) {
            // Absolute path relative to domain root
            $new_url = $base_url . ltrim($url, '/');
        } else {
            // Relative path
            $new_url = $base_url . $url;
        }

        // 4. Ensure we haven't double-slashed anything
        $new_url = preg_replace('#([^:])//+#', '$1/', $new_url);

        return $attr . '="' . $new_url . '"';
    }, $content);

    return $content;
}

// Function to fix malformed HTML comments
function fix_html_comments($content) {
    // Fix ONLY comments that end with -> (malformed)
    // Won't touch comments that already have -- in their closing
    return preg_replace('/<!--([^>]*?)(?<!-)->(?!-)/', '<!--$1-->', $content);
}

// Function to extract head content
function extract_head_content($dom, $base_url) {
    $head = $dom->getElementsByTagName('head')->item(0);
    if (!$head) {
        log_tnt_message("Head element not found");
        return '';
    }

    $content = '';
    foreach ($head->childNodes as $node) {
        if ($node->nodeName === 'meta' && strcasecmp($node->getAttribute('name'), 'description') === 0) {
            $content .= '<!-- ' . trim($dom->saveHTML($node)) . ' -->' . PHP_EOL;
        } elseif (!($node->nodeName === 'meta' && (
                $node->getAttribute('http-equiv') === 'Content-Type' ||
                $node->getAttribute('name') === 'viewport')) &&
            $node->nodeName !== 'title' &&
            !($node->nodeName === 'link' && $node->getAttribute('rel') === 'canonical')
        ) {
            $content .= $dom->saveHTML($node);
        } else {
            $content .= '<!-- ' . trim($dom->saveHTML($node)) . ' -->' . PHP_EOL;
        }
    }
    $content .= '</head>';  // Ensure we capture the closing head tag
    
    // Fix any malformed comments before adjusting paths
    $content = fix_html_comments($content);
    return adjust_paths($content, $base_url);
}

// Function to extract header content
function extract_header_content($dom, $base_url) {
    $header = $dom->getElementsByTagName('header')->item(0);
    if (!$header) {
        log_tnt_message("Header element not found");
        return '';
    }

    $content = '<header>' . PHP_EOL;
    foreach ($header->childNodes as $node) {
        $content .= $dom->saveHTML($node);
    }
    $content .= '</header>' . PHP_EOL;
    
    // Fix any malformed comments before adjusting paths
    $content = fix_html_comments($content);
    return adjust_paths($content, $base_url);
}

// Function to extract footer content
function extract_footer_content($dom, $base_url) {
    $footer = $dom->getElementsByTagName('footer')->item(0);
    if (!$footer) {
        log_tnt_message("Footer element not found");
        return '<!-- Footer not found -->';
    }

    $content = '<footer>' . PHP_EOL;
    foreach ($footer->childNodes as $node) {
        $content .= $dom->saveHTML($node);
    }
    $content .= '</footer>' . PHP_EOL;

    // Additional content after the footer
    $body = $dom->getElementsByTagName('body')->item(0);
    $after_footer = false;
    foreach ($body->childNodes as $node) {
        if ($node->nodeName === 'footer') {
            $after_footer = true;
        }
        if ($after_footer && $node->nodeName !== 'footer') {
            $content .= $dom->saveHTML($node);
        }
    }
    
    // Fix any malformed comments before adjusting paths
    $content = fix_html_comments($content);
    return adjust_paths($content, $base_url);
}

// Function to update options with fetched content
function update_content_options() {
    $domain = get_option('tnt_domain', 'https://www.example.com');
    $url = $domain . '/index.html'; // Fetching the specific index.html page
    $dom = fetch_static_site_content($url);

    if ($dom === null) {
        log_tnt_message("Failed to update content: DOM is null");
        return;
    }

    $head_content = extract_head_content($dom, $domain);
    $header_content = extract_header_content($dom, $domain);
    $footer_content = extract_footer_content($dom, $domain);

    update_option('custom_head_content', $head_content);
    update_option('custom_header_content', $header_content);
    update_option('custom_footer_content', $footer_content);

    inject_content_into_files($head_content, $header_content, $footer_content);

    log_tnt_message("Content updated successfully from $url");
}
add_action('admin_post_tnt_manual_update', 'update_content_options');

// Logging function
function log_tnt_message($message) {
    $log = get_option('tnt_log', []);
    $log[] = ['time' => current_time('mysql'), 'message' => $message];
    update_option('tnt_log', $log);
}

// Add admin menu page
add_action('admin_menu', 'tnt_custom_header_footer_admin_menu');
function tnt_custom_header_footer_admin_menu() {
    add_menu_page(
        __('Header/Footer Updater', 'tnt-header-footer'),
        __('Header/Footer Updater', 'tnt-header-footer'),
        'manage_options',
        'tnt-header-footer-updater',
        'tnt_custom_header_footer_page',
        'dashicons-update'
    );
}

// Admin page content
function tnt_custom_header_footer_page() {
    // Verify user capabilities
    if (!current_user_can('manage_options')) {
        wp_die(__('You do not have sufficient permissions to access this page.', 'tnt-header-footer'));
    }

    // Handle form submissions
    if ($_SERVER['REQUEST_METHOD'] === 'POST') {
        // Verify nonce for all POST actions
        if (!isset($_POST['tnt_nonce']) || !wp_verify_nonce($_POST['tnt_nonce'], 'tnt_header_footer_action')) {
            wp_die(__('Security check failed. Please try again.', 'tnt-header-footer'));
        }

        if (isset($_POST['tnt_save_header_footer'])) {
            $domain = isset($_POST['tnt_domain']) ? esc_url_raw($_POST['tnt_domain']) : '';
            if (empty($domain)) {
                add_settings_error(
                    'tnt_messages',
                    'tnt_domain_empty',
                    __('Domain cannot be empty.', 'tnt-header-footer'),
                    'error'
                );
            } else {
                update_option('tnt_domain', $domain);
                update_content_options();
                add_settings_error(
                    'tnt_messages',
                    'tnt_content_updated',
                    __('Content updated successfully.', 'tnt-header-footer'),
                    'updated'
                );
            }
        }

        if (isset($_POST['tnt_manual_update'])) {
            update_content_options();
            add_settings_error(
                'tnt_messages',
                'tnt_manual_update',
                __('Manual update triggered successfully.', 'tnt-header-footer'),
                'updated'
            );
        }
    }

    // Get current values
    $domain = get_option('tnt_domain', '');
    $head_content = get_option('custom_head_content', '');
    $header_content = get_option('custom_header_content', '');
    $footer_content = get_option('custom_footer_content', '');
    $log = get_option('tnt_log', []);

    // Display any error/update messages
    settings_errors('tnt_messages');

    // Reverse the log order
    $log = array_reverse($log);

    ?>
    <style>
        .tnt-top-section {
            background-color: white;
            padding: 20px;
        }
        .tnt-main-content {
            background-color: #f0f0f0;
            padding: 20px;
        }
        .tnt-log-table {
            display: none;
        }
        .accordion {
            cursor: pointer;
            padding: 10px;
            width: 100%;
            text-align: left;
            border: none;
            outline: none;
            transition: 0.4s;
            font-size: 16px;
            display: flex;
            align-items: center;
            justify-content: space-between;
            background-color: white;
            border: 1px solid #ccc;
            margin-bottom: 5px;
        }
        .panel {
            padding: 0 18px;
            display: none;
            background-color: white;
            overflow: hidden;
        }
        .accordion:after {
            content: '\25BC'; /* Down arrow */
            font-size: 13px;
            transition: transform 0.4s;
        }
        .accordion.active:after {
            transform: rotate(180deg); /* Up arrow */
        }
        .wrap h1 {
            font-weight: bold;
        }
    </style>
    <div class="wrap">
        <h1><?php echo esc_html__('Header/Footer Updater', 'tnt-header-footer'); ?></h1>
        
        <form method="post" action="">
            <?php wp_nonce_field('tnt_header_footer_action', 'tnt_nonce'); ?>
            
            <div class="tnt-domain-section">
                <h2><?php echo esc_html__('Domain Configuration', 'tnt-header-footer'); ?></h2>
                <div class="form-field">
                    <label for="tnt_domain"><?php echo esc_html__('Domain URL', 'tnt-header-footer'); ?></label>
                    <input type="url" 
                           id="tnt_domain"
                           name="tnt_domain" 
                           value="<?php echo esc_attr($domain); ?>" 
                           class="regular-text" 
                           placeholder="https://www.example.com"
                           required
                           pattern="https?://.+"
                    />
                    <p class="description">
                        <?php echo esc_html__('Enter the full domain URL starting with http:// or https://', 'tnt-header-footer'); ?>
                    </p>
                </div>
                
                <p class="submit">
                    <?php
                    submit_button(
                        __('Create Theme', 'tnt-header-footer'),
                        'primary',
                        'tnt_save_header_footer',
                        false
                    );
                    echo '&nbsp;';
                    submit_button(
                        __('Update Theme', 'tnt-header-footer'),
                        'secondary',
                        'tnt_manual_update',
                        false
                    );
                    ?>
                </p>
            </div>
        </form>

        <div class="tnt-main-content">
            <h2>Head Content <button type="button" class="button button-secondary" onclick="copyToClipboard('tnt_head_content')">Copy Head</button></h2>
            <textarea id="tnt_head_content" name="tnt_head_content" rows="20" cols="100"><?php echo esc_textarea($head_content); ?></textarea>
            <h2>Header Content <button type="button" class="button button-secondary" onclick="copyToClipboard('tnt_header_content')">Copy Header</button></h2>
            <textarea id="tnt_header_content" name="tnt_header_content" rows="20" cols="100"><?php echo esc_textarea($header_content); ?></textarea>
            <h2>Footer Content <button type="button" class="button button-secondary" onclick="copyToClipboard('tnt_footer_content')">Copy Footer</button></h2>
            <textarea id="tnt_footer_content" name="tnt_footer_content" rows="20" cols="100"><?php echo esc_textarea($footer_content); ?></textarea>
        </div>
        <h2>Update Log <span style="cursor: pointer;" onclick="toggleLogVisibility()">&#x25BC;</span></h2>
        <table class="widefat">
            <thead>
                <tr>
                    <th>Date and Time</th>
                    <th>Message</th>
                </tr>
            </thead>
        </table>
        <table class="widefat tnt-log-table">
            <tbody>
                <?php foreach ($log as $entry) : ?>
                    <tr>
                        <td><?php echo esc_html($entry['time']); ?></td>
                        <td><?php echo esc_html($entry['message']); ?></td>
                    </tr>
                <?php endforeach; ?>
            </tbody>
        </table>
    </div>
    <script>
        function copyToClipboard(elementId) {
            var copyText = document.getElementById(elementId);
            copyText.select();
            copyText.setSelectionRange(0, 99999); // For mobile devices
            document.execCommand("copy");
            alert("Copied the text: " + copyText.value);
        }

        function toggleLogVisibility() {
            var logTable = document.querySelector('.tnt-log-table');
            if (logTable.style.display === 'none' || logTable.style.display === '') {
                logTable.style.display = 'table';
            } else {
                logTable.style.display = 'none';
            }
        }

        var acc = document.getElementsByClassName("accordion");
        var i;

        for (i = 0; i < acc.length; i++) {
            acc[i].addEventListener("click", function() {
                this.classList.toggle("active");
                var panel = this.nextElementSibling;
                if (panel.style.display === "block") {
                    panel.style.display = "none";
                } else {
                    panel.style.display = "block";
                }
            });
        }
    </script>
    <?php
}
?>
